'use strict';

/**
 * LIBRARIES
 */

var dashButton = require( 'node-dash-button' );
var request    = require( 'request' );

/**
 * CONFIGURATION & EVENT BINDING
 */

var config = require( './config.json' );

Object.keys( config.buttons ).forEach( function( index ) {
    var buttonConfig = config.buttons[index];

    var button = dashButton( buttonConfig.mac, config.arp_interface );

    button.on( 'detected', buttonActivated );
} );

var homeAssistantUrl = config.home_assistant_proto + '://' + config.home_assistant_host + ':' + config.home_assistant_port + '/api/';

/**
 * BUTTON FUNCTIONALITY
 */

/**
 * Make HTTP(s) requests to Home Assistant to change SHM's status
 */
function buttonActivated( mac ) {
    var req = {
        url: homeAssistantUrl + 'states/' + config.home_assistant_alarm_entity_id,
        headers: {
            'Content-Type': 'application/json',
            'x-ha-access': config.home_assistant_pass
        }
    };

    // First, check the current status so we can toggle
    request( req, function( err, res, body ) {
        // Handle error states
        if ( err ) {
            console.error( err );
            return;
        }

        if ( 200 !== res.statusCode ) {
            console.error( res.statusCode );
            return;
        }

        // HA only deals with JSON
        body = JSON.parse( body );

        // Identify service URL based on current state
        var serviceBase = 'services/alarm_control_panel/';

        if ( 'disarmed' === body.state ) {
            req.url = homeAssistantUrl + serviceBase + 'alarm_arm_home';
        } else if ( 'armed_home' === body.state ) {
            req.url = homeAssistantUrl + serviceBase + 'alarm_disarm';
        } else {
            console.error( 'Invalid request based on current SHM status' );
            return;
        }

        // Build new request to update state
        req.method = 'POST';
        req.body   = JSON.stringify( {
            'entity_id': config.home_assistant_alarm_entity_id
        } );

        // Make request to change alarm status
        request( req, function( err, res, body ) {
            // Handle error states
            if ( err ) {
                console.error( err );
                return;
            }

            if ( 200 !== res.statusCode ) {
                console.error( res.statusCode );
                return;
            }

            // Nothing more to do
            // services endpoints don't return anything useful
            return true;
        } );
    } );
}
